# define F_CPU 10000000

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include "Zeichensatz_neu.h"

void transmit(unsigned char, char);
unsigned char* charToGraph(char);



enum ScrollStyles		// Anzeigestyle
{
	SWING = 0,
	SCROLL = 1
};


char* Text = "   ZWLF BOXKMPFER JAGEN VIKTOR QUER BER DEN GROEN SYLTER DEICH | zwlf boxkmpfer jagen viktor quer ber den groen sylter deich   "; // Dieser Text wird angezeigt


static int TextLen = 0;

void static Tempo()		//Geschwindigkeit mit dem der Text durchluft
{
	_delay_ms(100);

}

void static Load_Low()			//Datenbernahme durch die 7219 wird vorbereitet
{	
	PORTB &= ~(1 << PB4);
}

void static Load_High()			//Daten werden von den 7219 bernommen
{
	PORTB |= (1 << PB4);
}

static int ColumnNumber()
{
	return (TextLen * 8);
}

void UpdateDisplay(char* Buffer) 						         // zeigt 24 Spalten aus Buffer auf dem Display an
{
	for (int Stelle = 0; Stelle < 3; Stelle++)
	{
																			
		for (int Spalte = 0; Spalte < 8; Spalte++)
		{
			int BufferPos = (Spalte * 7) + Spalte + Stelle; 
				
			transmit(0x08 - ((Stelle) * 0x01), Buffer[BufferPos]);
		}
		Load_Low();
		asm volatile("nop");
		asm volatile("nop");
		asm volatile("nop");
		asm volatile("nop");
		Load_High();
	}
}

unsigned char* charToGraph(char c)                                   //holt Char aus Zeichentabelle
{
	return Zeichentabelle[(int)c];
}

char ColumnFromText(int c)                                           // holt Spalte c aus Text
{
	if (c == 0) { return charToGraph(Text[0])[0]; }

	int charPos = c / 8;
	int linePos = c % 8;

	return pgm_read_byte(&charToGraph(Text[charPos])[linePos]);
}

void TestDisplay(int Style)                                         // zeigt Text an
{
	TextLen = strlen(Text);
	int maxOffset = ColumnNumber() - 24;
	while (1)
	{
		char buf[24];
		
		if (Style == 0)
		{
			for (int off = 0; off < maxOffset; off++)
			{
				for (int i = 0; i < 24; i++)
				{
					buf[i] = ColumnFromText(i + off);
				}
				UpdateDisplay(buf);
				Tempo();
			}

			for (int off = 0; off < maxOffset; off++)
			{
				for (int i = 0; i < 24; i++)
				{
					buf[i] = ColumnFromText(i + (maxOffset - off));
				}
				UpdateDisplay(buf);
				Tempo();
			}
		}
		else if (Style == 1)
		{
			for (int off = 0; off < ColumnNumber(); off++)
			{
				for (int i = 0; i < 24; i++)
				{
					if (i+off < ColumnNumber())
					{
						buf[i] = ColumnFromText(i + off);
					}
					else if (i+off >= ColumnNumber())
					{
						int newColumn = (i + off) % ColumnNumber();
						buf[i] = ColumnFromText(newColumn);
					}
				}

				UpdateDisplay(buf);
				Tempo();

			}
		}
	}
}


void static SPI_MasterInit(void) {		/* SPI Schnittstelle initialisieren*/

	DDRB = (1<<PB4)|(1<<PB5)|(1<<PB7);    	// set PB4(SS), PB5 (MOSI) and PB7 (SCK) output, all others input

  	SPCR = (1<<MSTR)|(1<<SPE);   				// enable Master, SPI, set clock rate fck/4  
  
  	PORTB |= (1 << PB7)|(1 << PB4);
}

void transmit(unsigned char addr, char data) {		/* Daten zu den 7219 schicken*/
  
  SPDR = addr;              				// Start transmission
  while(!(SPSR & (1<<SPIF)));       		// Wait for transmission complete
	
  asm volatile("nop");
 
  SPDR = data;              				// Start transmission
  while(!(SPSR & (1<<SPIF)));      			// Wait for transmission complete
}

void static max7219_Init(void)				// 7219 werden initialisiert
{
  Load_Low();
  for (int i = 0; i < 3; i++)
  {
	transmit(0x0C,0x01);  					// normal mode  transmit(0x0C,0x01);
  }
  Load_High();

  Load_Low();
  for (int i = 0; i < 3; i++)
  {
	transmit(0x0A,0x01);  					// set intensity 0x00 - 0x0F
  }
  Load_High();

  Load_Low();
  for (int i = 0; i < 3; i++)
  {
	transmit(0x0B,0x07);  					// scan digits 0-7 Achtung, damit ist eine Stelle gemeint
  }
  Load_High();

  Load_Low();
  for (int i = 0; i < 3; i++)
  {
	transmit(0x09,0x00); 					// no decoding
  }
  Load_High();
}


int main()
{
	Zeichentabelle_Initialisieren();
	SPI_MasterInit();
	_delay_us(10);
	max7219_Init();
	
	char Test[24];

	for (int i = 0; i < 24; i++)
	{
		Test[i] = 0x00;
	}
	
	UpdateDisplay(Test);

	_delay_ms(1000);
	
	TestDisplay(SCROLL);    // bei 'SCROLL' luft der Text von Rechts nach Links durch und bei 'SWING' luft der Text von Rechts nach Links und wieder zurck.
	
 	return 0;
}

